@@ -1,27 +1,51 @@
-use std::ffi::OsStr;
-use std::path::{Path, PathBuf};
-use std::{env, fs};
+use std::{
+    env,
+    ffi::OsStr,
+    fs,
+    path::{Path, PathBuf},
+    time::SystemTime,
+};
 
 fn main() {
     if let Some(git_sha) = read_git_sha() {
-        println!("cargo:rustc-env={}={}", "BUILD_SHA", git_sha);
+        println!("cargo:rustc-env=BUILD_SHA={git_sha}");
     }
 
     if web_playground_files_present() {
-        println!("cargo:rustc-cfg={}", "TREE_SITTER_EMBED_WASM_BINDING");
+        println!("cargo:rustc-cfg=TREE_SITTER_EMBED_WASM_BINDING");
     }
 
-    let rust_binding_version = read_rust_binding_version();
-    println!(
-        "cargo:rustc-env={}={}",
-        "RUST_BINDING_VERSION", rust_binding_version,
-    );
+    let build_time = SystemTime::now()
+        .duration_since(SystemTime::UNIX_EPOCH)
+        .unwrap()
+        .as_secs_f64();
+    println!("cargo:rustc-env=BUILD_TIME={build_time}");
 
-    let emscripten_version = fs::read_to_string("emscripten-version").unwrap();
-    println!(
-        "cargo:rustc-env={}={}",
-        "EMSCRIPTEN_VERSION", emscripten_version,
-    );
+    #[cfg(any(
+        target_os = "linux",
+        target_os = "android",
+        target_os = "freebsd",
+        target_os = "openbsd",
+        target_os = "netbsd",
+        target_os = "dragonfly",
+    ))]
+    {
+        let out_dir = PathBuf::from(env::var("OUT_DIR").unwrap()).join("dynamic-symbols.txt");
+        std::fs::write(
+            &out_dir,
+            "{
+                ts_current_malloc;
+                ts_current_calloc;
+                ts_current_realloc;
+                ts_current_free;
+            };",
+        )
+        .unwrap();
+        println!(
+            "cargo:rustc-link-arg=-Wl,--dynamic-list={}",
+            out_dir.display()
+        );
+    }
 }
 
 fn web_playground_files_present() -> bool {
@@ -42,7 +66,8 @@ fn read_git_sha() -> Option<String> {
         git_path = repo_path.join(".git");
         if git_path.exists() {
             break;
-        } else if !repo_path.pop() {
+        }
+        if !repo_path.pop() {
             return None;
         }
     }
@@ -57,10 +82,10 @@ fn read_git_sha() -> Option<String> {
     }
     let git_head_path = git_dir_path.join("HEAD");
     if let Some(path) = git_head_path.to_str() {
-        println!("cargo:rerun-if-changed={}", path);
+        println!("cargo:rerun-if-changed={path}");
     }
     if let Ok(mut head_content) = fs::read_to_string(&git_head_path) {
-        if head_content.ends_with("\n") {
+        if head_content.ends_with('\n') {
             head_content.pop();
         }
 
@@ -71,13 +96,12 @@ fn read_git_sha() -> Option<String> {
                 // Go to real non-worktree gitdir
                 let git_dir_path = git_dir_path
                     .parent()
-                    .map(|p| {
+                    .and_then(|p| {
                         p.file_name()
                             .map(|n| n == OsStr::new("worktrees"))
                             .and_then(|x| x.then(|| p.parent()))
                     })
                     .flatten()
-                    .flatten()
                     .unwrap_or(&git_dir_path);
 
                 let file = git_dir_path.join(&head_content);
@@ -90,7 +114,7 @@ fn read_git_sha() -> Option<String> {
                             if let Some((hash, r#ref)) = line.split_once(' ') {
                                 if r#ref == head_content {
                                     if let Some(path) = packed_refs.to_str() {
-                                        println!("cargo:rerun-if-changed={}", path);
+                                        println!("cargo:rerun-if-changed={path}");
                                     }
                                     return Some(hash.to_string());
                                 }
@@ -101,26 +125,15 @@ fn read_git_sha() -> Option<String> {
                 }
             };
             if let Some(path) = ref_filename.to_str() {
-                println!("cargo:rerun-if-changed={}", path);
+                println!("cargo:rerun-if-changed={path}");
             }
             return fs::read_to_string(&ref_filename).ok();
         }
         // If we're on a detached commit, then the `HEAD` file itself contains the sha.
-        else if head_content.len() == 40 {
+        if head_content.len() == 40 {
             return Some(head_content);
         }
     }
 
     None
 }
-
-fn read_rust_binding_version() -> String {
-    let path = "Cargo.toml";
-    let text = fs::read_to_string(path).unwrap();
-    let cargo_toml = toml::from_str::<toml::Value>(text.as_ref()).unwrap();
-    cargo_toml["dependencies"]["tree-sitter"]["version"]
-        .as_str()
-        .unwrap()
-        .trim_matches('"')
-        .to_string()
-}
