@@ -1,31 +1,36 @@
-use super::item::{ParseItem, ParseItemSet, ParseItemSetCore};
-use super::item_set_builder::ParseItemSetBuilder;
-use crate::generate::grammars::PrecedenceEntry;
-use crate::generate::grammars::{
-    InlinedProductionMap, LexicalGrammar, SyntaxGrammar, VariableType,
+use std::{
+    cmp::Ordering,
+    collections::{BTreeMap, HashMap, HashSet, VecDeque},
+    fmt::Write,
+    hash::BuildHasherDefault,
 };
-use crate::generate::node_types::VariableInfo;
-use crate::generate::rules::{Associativity, Precedence, Symbol, SymbolType, TokenSet};
-use crate::generate::tables::{
-    FieldLocation, GotoAction, ParseAction, ParseState, ParseStateId, ParseTable, ParseTableEntry,
-    ProductionInfo, ProductionInfoId,
-};
-use anyhow::{anyhow, Result};
-use std::cmp::Ordering;
-use std::collections::{BTreeMap, HashMap, HashSet, VecDeque};
-use std::fmt::Write;
-use std::hash::BuildHasherDefault;
-use std::u32;
 
+use anyhow::{anyhow, Result};
 use indexmap::{map::Entry, IndexMap};
 use rustc_hash::FxHasher;
 
+use super::{
+    item::{ParseItem, ParseItemSet, ParseItemSetCore},
+    item_set_builder::ParseItemSetBuilder,
+};
+use crate::generate::{
+    grammars::{
+        InlinedProductionMap, LexicalGrammar, PrecedenceEntry, SyntaxGrammar, VariableType,
+    },
+    node_types::VariableInfo,
+    rules::{Associativity, Precedence, Symbol, SymbolType, TokenSet},
+    tables::{
+        FieldLocation, GotoAction, ParseAction, ParseState, ParseStateId, ParseTable,
+        ParseTableEntry, ProductionInfo, ProductionInfoId,
+    },
+};
+
 // For conflict reporting, each parse state is associated with an example
 // sequence of symbols that could lead to that parse state.
 type SymbolSequence = Vec<Symbol>;
 
 type AuxiliarySymbolSequence = Vec<AuxiliarySymbolInfo>;
-pub(crate) type ParseStateInfo<'a> = (SymbolSequence, ParseItemSet<'a>);
+pub type ParseStateInfo<'a> = (SymbolSequence, ParseItemSet<'a>);
 
 #[derive(Clone)]
 struct AuxiliarySymbolInfo {
@@ -51,12 +56,13 @@ struct ParseTableBuilder<'a> {
     item_set_builder: ParseItemSetBuilder<'a>,
     syntax_grammar: &'a SyntaxGrammar,
     lexical_grammar: &'a LexicalGrammar,
-    variable_info: &'a Vec<VariableInfo>,
+    variable_info: &'a [VariableInfo],
     core_ids_by_core: HashMap<ParseItemSetCore<'a>, usize>,
     state_ids_by_item_set: IndexMap<ParseItemSet<'a>, ParseStateId, BuildHasherDefault<FxHasher>>,
     parse_state_info_by_id: Vec<ParseStateInfo<'a>>,
     parse_state_queue: VecDeque<ParseStateQueueEntry>,
     non_terminal_extra_states: Vec<(Symbol, usize)>,
+    actual_conflicts: HashSet<Vec<Symbol>>,
     parse_table: ParseTable,
 }
 
@@ -74,14 +80,10 @@ impl<'a> ParseTableBuilder<'a> {
         self.add_parse_state(
             &Vec::new(),
             &Vec::new(),
-            ParseItemSet::with(
-                [(
-                    ParseItem::start(),
-                    [Symbol::end()].iter().cloned().collect(),
-                )]
-                .iter()
-                .cloned(),
-            ),
+            ParseItemSet::with(std::iter::once((
+                ParseItem::start(),
+                std::iter::once(&Symbol::end()).copied().collect(),
+            ))),
         );
 
         // Compute the possible item sets for non-terminal extras.
@@ -96,7 +98,7 @@ impl<'a> ParseTableBuilder<'a> {
             for production in &variable.productions {
                 non_terminal_extra_item_sets_by_first_terminal
                     .entry(production.first_symbol().unwrap())
-                    .or_insert(ParseItemSet::default())
+                    .or_insert_with(ParseItemSet::default)
                     .insert(
                         ParseItem {
                             variable_index: extra_non_terminal.index as u32,
@@ -104,9 +106,8 @@ impl<'a> ParseTableBuilder<'a> {
                             step_index: 1,
                             has_preceding_inherited_fields: false,
                         },
-                        &[Symbol::end_of_nonterminal_extra()]
-                            .iter()
-                            .cloned()
+                        &std::iter::once(&Symbol::end_of_nonterminal_extra())
+                            .copied()
                             .collect(),
                     );
             }
@@ -128,10 +129,24 @@ impl<'a> ParseTableBuilder<'a> {
                 self.parse_state_info_by_id[entry.state_id].0.clone(),
                 entry.preceding_auxiliary_symbols,
                 entry.state_id,
-                item_set,
+                &item_set,
             )?;
         }
 
+        if !self.actual_conflicts.is_empty() {
+            println!("Warning: unnecessary conflicts");
+            for conflict in &self.actual_conflicts {
+                println!(
+                    "  {}",
+                    conflict
+                        .iter()
+                        .map(|symbol| format!("`{}`", self.symbol_name(symbol)))
+                        .collect::<Vec<_>>()
+                        .join(", ")
+                );
+            }
+        }
+
         Ok((self.parse_table, self.parse_state_info_by_id))
     }
 
@@ -180,7 +195,7 @@ impl<'a> ParseTableBuilder<'a> {
         mut preceding_symbols: SymbolSequence,
         mut preceding_auxiliary_symbols: Vec<AuxiliarySymbolInfo>,
         state_id: ParseStateId,
-        item_set: ParseItemSet<'a>,
+        item_set: &ParseItemSet<'a>,
     ) -> Result<()> {
         let mut terminal_successors = BTreeMap::new();
         let mut non_terminal_successors = BTreeMap::new();
@@ -203,7 +218,7 @@ impl<'a> ParseTableBuilder<'a> {
                     // for conflict resolution.
                     if variable.is_auxiliary() {
                         preceding_auxiliary_symbols
-                            .push(self.get_auxiliary_node_info(&item_set, next_symbol));
+                            .push(self.get_auxiliary_node_info(item_set, next_symbol));
                     }
 
                     // For most parse items, the symbols associated with the preceding children
@@ -223,12 +238,12 @@ impl<'a> ParseTableBuilder<'a> {
 
                     non_terminal_successors
                         .entry(next_symbol)
-                        .or_insert_with(|| ParseItemSet::default())
+                        .or_insert_with(ParseItemSet::default)
                         .insert(successor, lookaheads);
                 } else {
                     terminal_successors
                         .entry(next_symbol)
-                        .or_insert_with(|| ParseItemSet::default())
+                        .or_insert_with(ParseItemSet::default)
                         .insert(successor, lookaheads);
                 }
             }
@@ -253,7 +268,7 @@ impl<'a> ParseTableBuilder<'a> {
                     let table_entry = self.parse_table.states[state_id]
                         .terminal_entries
                         .entry(lookahead)
-                        .or_insert_with(|| ParseTableEntry::new());
+                        .or_insert_with(ParseTableEntry::new);
                     let reduction_info = reduction_infos.entry(lookahead).or_default();
 
                     // While inserting Reduce actions, eagerly resolve conflicts related
@@ -263,7 +278,7 @@ impl<'a> ParseTableBuilder<'a> {
                         table_entry.actions.push(action);
                     } else {
                         match Self::compare_precedence(
-                            &self.syntax_grammar,
+                            self.syntax_grammar,
                             precedence,
                             &[symbol],
                             &reduction_info.precedence,
@@ -283,7 +298,7 @@ impl<'a> ParseTableBuilder<'a> {
                         }
                     }
 
-                    reduction_info.precedence = precedence.clone();
+                    reduction_info.precedence.clone_from(precedence);
                     if let Err(i) = reduction_info.symbols.binary_search(&symbol) {
                         reduction_info.symbols.insert(i, symbol);
                     }
@@ -296,7 +311,7 @@ impl<'a> ParseTableBuilder<'a> {
             }
         }
 
-        // Having computed the the successor item sets for each symbol, add a new
+        // Having computed the successor item sets for each symbol, add a new
         // parse state for each of these item sets, and add a corresponding Shift
         // action to this state.
         for (symbol, next_item_set) in terminal_successors {
@@ -318,7 +333,7 @@ impl<'a> ParseTableBuilder<'a> {
             }
 
             entry
-                .or_insert_with(|| ParseTableEntry::new())
+                .or_insert_with(ParseTableEntry::new)
                 .actions
                 .push(ParseAction::Shift {
                     state: next_state_id,
@@ -346,7 +361,7 @@ impl<'a> ParseTableBuilder<'a> {
         // * fail, terminating the parser generation process
         for symbol in lookaheads_with_conflicts.iter() {
             self.handle_conflict(
-                &item_set,
+                item_set,
                 state_id,
                 &preceding_symbols,
                 &preceding_auxiliary_symbols,
@@ -429,7 +444,7 @@ impl<'a> ParseTableBuilder<'a> {
         item_set: &ParseItemSet,
         state_id: ParseStateId,
         preceding_symbols: &SymbolSequence,
-        preceding_auxiliary_symbols: &Vec<AuxiliarySymbolInfo>,
+        preceding_auxiliary_symbols: &[AuxiliarySymbolInfo],
         conflicting_lookahead: Symbol,
         reduction_info: &ReductionInfo,
     ) -> Result<()> {
@@ -445,33 +460,31 @@ impl<'a> ParseTableBuilder<'a> {
         // REDUCE-REDUCE conflicts where all actions have the *same*
         // precedence, and there can still be SHIFT/REDUCE conflicts.
         let mut considered_associativity = false;
-        let mut shift_precedence: Vec<(&Precedence, Symbol)> = Vec::new();
+        let mut shift_precedence = Vec::<(&Precedence, Symbol)>::new();
         let mut conflicting_items = HashSet::new();
         for (item, lookaheads) in &item_set.entries {
             if let Some(step) = item.step() {
-                if item.step_index > 0 {
-                    if self
+                if item.step_index > 0
+                    && self
                         .item_set_builder
                         .first_set(&step.symbol)
                         .contains(&conflicting_lookahead)
-                    {
-                        if item.variable_index != u32::MAX {
-                            conflicting_items.insert(item);
-                        }
+                {
+                    if item.variable_index != u32::MAX {
+                        conflicting_items.insert(item);
+                    }
 
-                        let p = (
-                            item.precedence(),
-                            Symbol::non_terminal(item.variable_index as usize),
-                        );
-                        if let Err(i) = shift_precedence.binary_search(&p) {
-                            shift_precedence.insert(i, p);
-                        }
+                    let p = (
+                        item.precedence(),
+                        Symbol::non_terminal(item.variable_index as usize),
+                    );
+                    if let Err(i) = shift_precedence.binary_search(&p) {
+                        shift_precedence.insert(i, p);
                     }
                 }
-            } else if lookaheads.contains(&conflicting_lookahead) {
-                if item.variable_index != u32::MAX {
-                    conflicting_items.insert(item);
-                }
+            } else if lookaheads.contains(&conflicting_lookahead) && item.variable_index != u32::MAX
+            {
+                conflicting_items.insert(item);
             }
         }
 
@@ -497,7 +510,7 @@ impl<'a> ParseTableBuilder<'a> {
             let mut shift_is_more = false;
             for p in shift_precedence {
                 match Self::compare_precedence(
-                    &self.syntax_grammar,
+                    self.syntax_grammar,
                     p.0,
                     &[p.1],
                     &reduction_info.precedence,
@@ -582,6 +595,7 @@ impl<'a> ParseTableBuilder<'a> {
             .expected_conflicts
             .contains(&actual_conflict)
         {
+            self.actual_conflicts.remove(&actual_conflict);
             return Ok(());
         }
 
@@ -590,15 +604,15 @@ impl<'a> ParseTableBuilder<'a> {
             write!(&mut msg, "  {}", self.symbol_name(symbol)).unwrap();
         }
 
-        write!(
+        writeln!(
             &mut msg,
-            "  •  {}  …\n\n",
+            "  •  {}  …\n",
             self.symbol_name(&conflicting_lookahead)
         )
         .unwrap();
-        write!(&mut msg, "Possible interpretations:\n\n").unwrap();
+        writeln!(&mut msg, "Possible interpretations:\n").unwrap();
 
-        let mut interpretions = conflicting_items
+        let mut interpretations = conflicting_items
             .iter()
             .map(|item| {
                 let mut line = String::new();
@@ -639,11 +653,10 @@ impl<'a> ParseTableBuilder<'a> {
 
                 let prec_line = if let Some(associativity) = associativity {
                     Some(format!(
-                        "(precedence: {}, associativity: {:?})",
-                        precedence, associativity
+                        "(precedence: {precedence}, associativity: {associativity:?})",
                     ))
                 } else if !precedence.is_none() {
-                    Some(format!("(precedence: {})", precedence))
+                    Some(format!("(precedence: {precedence})"))
                 } else {
                     None
                 };
@@ -652,13 +665,13 @@ impl<'a> ParseTableBuilder<'a> {
             })
             .collect::<Vec<_>>();
 
-        let max_interpretation_length = interpretions
+        let max_interpretation_length = interpretations
             .iter()
             .map(|i| i.0.chars().count())
             .max()
             .unwrap();
-        interpretions.sort_unstable();
-        for (i, (line, prec_suffix)) in interpretions.into_iter().enumerate() {
+        interpretations.sort_unstable();
+        for (i, (line, prec_suffix)) in interpretations.into_iter().enumerate() {
             write!(&mut msg, "  {}:", i + 1).unwrap();
             msg += &line;
             if let Some(prec_suffix) = prec_suffix {
@@ -672,7 +685,7 @@ impl<'a> ParseTableBuilder<'a> {
         }
 
         let mut resolution_count = 0;
-        write!(&mut msg, "\nPossible resolutions:\n\n").unwrap();
+        writeln!(&mut msg, "\nPossible resolutions:\n").unwrap();
         let mut shift_items = Vec::new();
         let mut reduce_items = Vec::new();
         for item in conflicting_items {
@@ -707,24 +720,22 @@ impl<'a> ParseTableBuilder<'a> {
         };
 
         if actual_conflict.len() > 1 {
-            if shift_items.len() > 0 {
+            if !shift_items.is_empty() {
                 resolution_count += 1;
                 write!(
                     &mut msg,
-                    "  {}:  Specify a higher precedence in",
-                    resolution_count
+                    "  {resolution_count}:  Specify a higher precedence in",
                 )
                 .unwrap();
                 list_rule_names(&mut msg, &shift_items);
-                write!(&mut msg, " than in the other rules.\n").unwrap();
+                writeln!(&mut msg, " than in the other rules.").unwrap();
             }
 
             for item in &reduce_items {
                 resolution_count += 1;
-                write!(
+                writeln!(
                     &mut msg,
-                    "  {}:  Specify a higher precedence in `{}` than in the other rules.\n",
-                    resolution_count,
+                    "  {resolution_count}:  Specify a higher precedence in `{}` than in the other rules.",
                     self.symbol_name(&Symbol::non_terminal(item.variable_index as usize))
                 )
                 .unwrap();
@@ -735,19 +746,17 @@ impl<'a> ParseTableBuilder<'a> {
             resolution_count += 1;
             write!(
                 &mut msg,
-                "  {}:  Specify a left or right associativity in",
-                resolution_count
+                "  {resolution_count}:  Specify a left or right associativity in",
             )
             .unwrap();
             list_rule_names(&mut msg, &reduce_items);
-            write!(&mut msg, "\n").unwrap();
+            writeln!(&mut msg).unwrap();
         }
 
         resolution_count += 1;
         write!(
             &mut msg,
-            "  {}:  Add a conflict for these rules: ",
-            resolution_count
+            "  {resolution_count}:  Add a conflict for these rules: ",
         )
         .unwrap();
         for (i, symbol) in actual_conflict.iter().enumerate() {
@@ -756,7 +765,7 @@ impl<'a> ParseTableBuilder<'a> {
             }
             write!(&mut msg, "`{}`", self.symbol_name(symbol)).unwrap();
         }
-        write!(&mut msg, "\n").unwrap();
+        writeln!(&mut msg).unwrap();
 
         Err(anyhow!(msg))
     }
@@ -789,7 +798,7 @@ impl<'a> ParseTableBuilder<'a> {
             // and to the default precedence, which is zero.
             (Precedence::Integer(l), Precedence::Integer(r)) if *l != 0 || *r != 0 => l.cmp(r),
             (Precedence::Integer(l), Precedence::None) if *l != 0 => l.cmp(&0),
-            (Precedence::None, Precedence::Integer(r)) if *r != 0 => 0.cmp(&r),
+            (Precedence::None, Precedence::Integer(r)) if *r != 0 => 0.cmp(r),
 
             // Named precedences can be compared to other named precedences.
             _ => grammar
@@ -856,7 +865,7 @@ impl<'a> ParseTableBuilder<'a> {
                 production_info
                     .field_map
                     .entry(field_name.clone())
-                    .or_insert(Vec::new())
+                    .or_default()
                     .push(FieldLocation {
                         index: i,
                         inherited: false,
@@ -869,11 +878,11 @@ impl<'a> ParseTableBuilder<'a> {
                     .is_visible()
             {
                 let info = &self.variable_info[step.symbol.index];
-                for (field_name, _) in &info.fields {
+                for field_name in info.fields.keys() {
                     production_info
                         .field_map
                         .entry(field_name.clone())
-                        .or_insert(Vec::new())
+                        .or_default()
                         .push(FieldLocation {
                             index: i,
                             inherited: true,
@@ -887,7 +896,7 @@ impl<'a> ParseTableBuilder<'a> {
         }
 
         if item.production.steps.len() > self.parse_table.max_aliased_production_length {
-            self.parse_table.max_aliased_production_length = item.production.steps.len()
+            self.parse_table.max_aliased_production_length = item.production.steps.len();
         }
 
         if let Some(index) = self
@@ -923,7 +932,7 @@ impl<'a> ParseTableBuilder<'a> {
 }
 
 fn populate_following_tokens(
-    result: &mut Vec<TokenSet>,
+    result: &mut [TokenSet],
     grammar: &SyntaxGrammar,
     inlines: &InlinedProductionMap,
     builder: &ParseItemSetBuilder,
@@ -934,7 +943,6 @@ fn populate_following_tokens(
         .flat_map(|v| &v.productions)
         .chain(&inlines.productions);
     let all_tokens = (0..result.len())
-        .into_iter()
         .map(Symbol::terminal)
         .collect::<TokenSet>();
     for production in productions {
@@ -953,17 +961,18 @@ fn populate_following_tokens(
             for entry in result.iter_mut() {
                 entry.insert(*extra);
             }
-            result[extra.index] = all_tokens.clone();
+            result[extra.index].clone_from(&all_tokens);
         }
     }
 }
 
-pub(crate) fn build_parse_table<'a>(
+pub fn build_parse_table<'a>(
     syntax_grammar: &'a SyntaxGrammar,
     lexical_grammar: &'a LexicalGrammar,
     inlines: &'a InlinedProductionMap,
-    variable_info: &'a Vec<VariableInfo>,
+    variable_info: &'a [VariableInfo],
 ) -> Result<(ParseTable, Vec<TokenSet>, Vec<ParseStateInfo<'a>>)> {
+    let actual_conflicts = syntax_grammar.expected_conflicts.iter().cloned().collect();
     let item_set_builder = ParseItemSetBuilder::new(syntax_grammar, lexical_grammar, inlines);
     let mut following_tokens = vec![TokenSet::new(); lexical_grammar.variables.len()];
     populate_following_tokens(
@@ -979,6 +988,7 @@ pub(crate) fn build_parse_table<'a>(
         item_set_builder,
         variable_info,
         non_terminal_extra_states: Vec::new(),
+        actual_conflicts,
         state_ids_by_item_set: IndexMap::default(),
         core_ids_by_core: HashMap::new(),
         parse_state_info_by_id: Vec::new(),
