@@ -1,15 +1,19 @@
-use super::token_conflicts::TokenConflictMap;
-use crate::generate::dedup::split_state_id_groups;
-use crate::generate::grammars::{LexicalGrammar, SyntaxGrammar, VariableType};
-use crate::generate::rules::{AliasMap, Symbol, TokenSet};
-use crate::generate::tables::{
-    GotoAction, ParseAction, ParseState, ParseStateId, ParseTable, ParseTableEntry,
+use std::{
+    collections::{HashMap, HashSet},
+    mem,
 };
-use log::info;
-use std::collections::{HashMap, HashSet};
-use std::mem;
 
-pub(crate) fn minimize_parse_table(
+use log::info;
+
+use super::token_conflicts::TokenConflictMap;
+use crate::generate::{
+    dedup::split_state_id_groups,
+    grammars::{LexicalGrammar, SyntaxGrammar, VariableType},
+    rules::{AliasMap, Symbol, TokenSet},
+    tables::{GotoAction, ParseAction, ParseState, ParseStateId, ParseTable, ParseTableEntry},
+};
+
+pub fn minimize_parse_table(
     parse_table: &mut ParseTable,
     syntax_grammar: &SyntaxGrammar,
     lexical_grammar: &LexicalGrammar,
@@ -67,9 +71,9 @@ impl<'a> Minimizer<'a> {
                             symbol,
                             ..
                         } => {
-                            if !self.simple_aliases.contains_key(&symbol)
-                                && !self.syntax_grammar.supertype_symbols.contains(&symbol)
-                                && !aliased_symbols.contains(&symbol)
+                            if !self.simple_aliases.contains_key(symbol)
+                                && !self.syntax_grammar.supertype_symbols.contains(symbol)
+                                && !aliased_symbols.contains(symbol)
                                 && self.syntax_grammar.variables[symbol.index].kind
                                     != VariableType::Named
                                 && (unit_reduction_symbol.is_none()
@@ -97,21 +101,22 @@ impl<'a> Minimizer<'a> {
             }
         }
 
-        for state in self.parse_table.states.iter_mut() {
+        for state in &mut self.parse_table.states {
             let mut done = false;
             while !done {
                 done = true;
                 state.update_referenced_states(|other_state_id, state| {
-                    if let Some(symbol) = unit_reduction_symbols_by_state.get(&other_state_id) {
-                        done = false;
-                        match state.nonterminal_entries.get(symbol) {
-                            Some(GotoAction::Goto(state_id)) => *state_id,
-                            _ => other_state_id,
-                        }
-                    } else {
-                        other_state_id
-                    }
-                })
+                    unit_reduction_symbols_by_state.get(&other_state_id).map_or(
+                        other_state_id,
+                        |symbol| {
+                            done = false;
+                            match state.nonterminal_entries.get(symbol) {
+                                Some(GotoAction::Goto(state_id)) => *state_id,
+                                _ => other_state_id,
+                            }
+                        },
+                    )
+                });
             }
         }
     }
@@ -198,7 +203,7 @@ impl<'a> Minimizer<'a> {
         &self,
         left_state: &ParseState,
         right_state: &ParseState,
-        group_ids_by_state_id: &Vec<ParseStateId>,
+        group_ids_by_state_id: &[ParseStateId],
     ) -> bool {
         for (token, left_entry) in &left_state.terminal_entries {
             if let Some(right_entry) = right_state.terminal_entries.get(token) {
@@ -223,15 +228,15 @@ impl<'a> Minimizer<'a> {
         }
 
         for token in right_state.terminal_entries.keys() {
-            if !left_state.terminal_entries.contains_key(token) {
-                if self.token_conflicts(
+            if !left_state.terminal_entries.contains_key(token)
+                && self.token_conflicts(
                     left_state.id,
                     right_state.id,
                     left_state.terminal_entries.keys(),
                     *token,
-                ) {
-                    return true;
-                }
+                )
+            {
+                return true;
             }
         }
 
@@ -242,7 +247,7 @@ impl<'a> Minimizer<'a> {
         &self,
         state1: &ParseState,
         state2: &ParseState,
-        group_ids_by_state_id: &Vec<ParseStateId>,
+        group_ids_by_state_id: &[ParseStateId],
     ) -> bool {
         for (token, entry1) in &state1.terminal_entries {
             if let ParseAction::Shift { state: s1, .. } = entry1.actions.last().unwrap() {
@@ -252,12 +257,10 @@ impl<'a> Minimizer<'a> {
                         let group2 = group_ids_by_state_id[*s2];
                         if group1 != group2 {
                             info!(
-                                "split states {} {} - successors for {} are split: {} {}",
+                                "split states {} {} - successors for {} are split: {s1} {s2}",
                                 state1.id,
                                 state2.id,
                                 self.symbol_name(token),
-                                s1,
-                                s2,
                             );
                             return true;
                         }
@@ -275,12 +278,10 @@ impl<'a> Minimizer<'a> {
                         let group2 = group_ids_by_state_id[*s2];
                         if group1 != group2 {
                             info!(
-                                "split states {} {} - successors for {} are split: {} {}",
+                                "split states {} {} - successors for {} are split: {s1} {s2}",
                                 state1.id,
                                 state2.id,
                                 self.symbol_name(symbol),
-                                s1,
-                                s2,
                             );
                             return true;
                         }
@@ -300,16 +301,14 @@ impl<'a> Minimizer<'a> {
         token: &Symbol,
         entry1: &ParseTableEntry,
         entry2: &ParseTableEntry,
-        group_ids_by_state_id: &Vec<ParseStateId>,
+        group_ids_by_state_id: &[ParseStateId],
     ) -> bool {
         // To be compatible, entries need to have the same actions.
         let actions1 = &entry1.actions;
         let actions2 = &entry2.actions;
         if actions1.len() != actions2.len() {
             info!(
-                "split states {} {} - differing action counts for token {}",
-                state_id1,
-                state_id2,
+                "split states {state_id1} {state_id2} - differing action counts for token {}",
                 self.symbol_name(token)
             );
             return true;
@@ -334,22 +333,15 @@ impl<'a> Minimizer<'a> {
                 let group2 = group_ids_by_state_id[*s2];
                 if group1 == group2 && is_repetition1 == is_repetition2 {
                     continue;
-                } else {
-                    info!(
-                        "split states {} {} - successors for {} are split: {} {}",
-                        state_id1,
-                        state_id2,
-                        self.symbol_name(token),
-                        s1,
-                        s2,
-                    );
-                    return true;
                 }
+                info!(
+                    "split states {state_id1} {state_id2} - successors for {} are split: {s1} {s2}",
+                    self.symbol_name(token),
+                );
+                return true;
             } else if action1 != action2 {
                 info!(
-                    "split states {} {} - unequal actions for {}",
-                    state_id1,
-                    state_id2,
+                    "split states {state_id1} {state_id2} - unequal actions for {}",
                     self.symbol_name(token),
                 );
                 return true;
@@ -367,10 +359,7 @@ impl<'a> Minimizer<'a> {
         new_token: Symbol,
     ) -> bool {
         if new_token == Symbol::end_of_nonterminal_extra() {
-            info!(
-                "split states {} {} - end of non-terminal extra",
-                left_id, right_id,
-            );
+            info!("split states {left_id} {right_id} - end of non-terminal extra",);
             return true;
         }
 
@@ -378,9 +367,7 @@ impl<'a> Minimizer<'a> {
         // existing lookahead tokens.
         if new_token.is_external() {
             info!(
-                "split states {} {} - external token {}",
-                left_id,
-                right_id,
+                "split states {left_id} {right_id} - external token {}",
                 self.symbol_name(&new_token),
             );
             return true;
@@ -395,9 +382,7 @@ impl<'a> Minimizer<'a> {
             .any(|external| external.corresponding_internal_token == Some(new_token))
         {
             info!(
-                "split states {} {} - internal/external token {}",
-                left_id,
-                right_id,
+                "split states {left_id} {right_id} - internal/external token {}",
                 self.symbol_name(&new_token),
             );
             return true;
@@ -405,27 +390,24 @@ impl<'a> Minimizer<'a> {
 
         // Do not add a token if it conflicts with an existing token.
         for token in existing_tokens {
-            if token.is_terminal() {
-                if !(self.syntax_grammar.word_token == Some(*token)
+            if token.is_terminal()
+                && !(self.syntax_grammar.word_token == Some(*token)
                     && self.keywords.contains(&new_token))
-                    && !(self.syntax_grammar.word_token == Some(new_token)
-                        && self.keywords.contains(token))
-                    && (self
+                && !(self.syntax_grammar.word_token == Some(new_token)
+                    && self.keywords.contains(token))
+                && (self
+                    .token_conflict_map
+                    .does_conflict(new_token.index, token.index)
+                    || self
                         .token_conflict_map
-                        .does_conflict(new_token.index, token.index)
-                        || self
-                            .token_conflict_map
-                            .does_match_same_string(new_token.index, token.index))
-                {
-                    info!(
-                        "split states {} {} - token {} conflicts with {}",
-                        left_id,
-                        right_id,
-                        self.symbol_name(&new_token),
-                        self.symbol_name(token),
-                    );
-                    return true;
-                }
+                        .does_match_same_string(new_token.index, token.index))
+            {
+                info!(
+                    "split states {left_id} {right_id} - token {} conflicts with {}",
+                    self.symbol_name(&new_token),
+                    self.symbol_name(token),
+                );
+                return true;
             }
         }
 
