@@ -1,13 +1,14 @@
-use std::char;
-use std::cmp::max;
-use std::cmp::Ordering;
-use std::collections::HashSet;
-use std::fmt;
-use std::mem::swap;
-use std::ops::Range;
+use std::{
+    char,
+    cmp::{max, Ordering},
+    fmt,
+    iter::ExactSizeIterator,
+    mem::{self, swap},
+    ops::{Range, RangeInclusive},
+};
 
 /// A set of characters represented as a vector of ranges.
-#[derive(Clone, PartialEq, Eq, Hash)]
+#[derive(Clone, Default, PartialEq, Eq, Hash)]
 pub struct CharacterSet {
     ranges: Vec<Range<u32>>,
 }
@@ -28,7 +29,7 @@ pub enum NfaState {
     },
 }
 
-#[derive(PartialEq, Eq)]
+#[derive(PartialEq, Eq, Default)]
 pub struct Nfa {
     pub states: Vec<NfaState>,
 }
@@ -47,40 +48,36 @@ pub struct NfaTransition {
     pub states: Vec<u32>,
 }
 
-impl Default for Nfa {
-    fn default() -> Self {
-        Self { states: Vec::new() }
-    }
-}
-
 const END: u32 = char::MAX as u32 + 1;
 
 impl CharacterSet {
     /// Create a character set with a single character.
-    pub fn empty() -> Self {
-        CharacterSet { ranges: Vec::new() }
+    pub const fn empty() -> Self {
+        Self { ranges: Vec::new() }
     }
 
     /// Create a character set with a given *inclusive* range of characters.
+    #[allow(clippy::single_range_in_vec_init)]
     pub fn from_range(mut first: char, mut last: char) -> Self {
         if first > last {
             swap(&mut first, &mut last);
         }
-        CharacterSet {
+        Self {
             ranges: vec![(first as u32)..(last as u32 + 1)],
         }
     }
 
     /// Create a character set with a single character.
+    #[allow(clippy::single_range_in_vec_init)]
     pub fn from_char(c: char) -> Self {
-        CharacterSet {
+        Self {
             ranges: vec![(c as u32)..(c as u32 + 1)],
         }
     }
 
     /// Create a character set containing all characters *not* present
     /// in this character set.
-    pub fn negate(mut self) -> CharacterSet {
+    pub fn negate(mut self) -> Self {
         let mut i = 0;
         let mut previous_end = 0;
         while i < self.ranges.len() {
@@ -110,14 +107,19 @@ impl CharacterSet {
         self
     }
 
-    pub fn add(mut self, other: &CharacterSet) -> Self {
+    pub fn add(mut self, other: &Self) -> Self {
         let mut index = 0;
         for range in &other.ranges {
-            index = self.add_int_range(index, range.start as u32, range.end as u32);
+            index = self.add_int_range(index, range.start, range.end);
         }
         self
     }
 
+    pub fn assign(&mut self, other: &Self) {
+        self.ranges.clear();
+        self.ranges.extend_from_slice(&other.ranges);
+    }
+
     fn add_int_range(&mut self, mut i: usize, start: u32, end: u32) -> usize {
         while i < self.ranges.len() {
             let range = &mut self.ranges[i];
@@ -143,7 +145,7 @@ impl CharacterSet {
         i
     }
 
-    pub fn does_intersect(&self, other: &CharacterSet) -> bool {
+    pub fn does_intersect(&self, other: &Self) -> bool {
         let mut left_ranges = self.ranges.iter();
         let mut right_ranges = other.ranges.iter();
         let mut left_range = left_ranges.next();
@@ -163,7 +165,7 @@ impl CharacterSet {
     /// Get the set of characters that are present in both this set
     /// and the other set. Remove those common characters from both
     /// of the operands.
-    pub fn remove_intersection(&mut self, other: &mut CharacterSet) -> CharacterSet {
+    pub fn remove_intersection(&mut self, other: &mut Self) -> Self {
         let mut intersection = Vec::new();
         let mut left_i = 0;
         let mut right_i = 0;
@@ -209,29 +211,28 @@ impl CharacterSet {
                         }
                     }
                 }
-                Ordering::Equal => {
-                    // [ L ]
-                    // [  R  ]
-                    if left.end < right.end {
-                        intersection.push(left.start..left.end);
-                        right.start = left.end;
-                        self.ranges.remove(left_i);
-                    }
-                    // [ L ]
-                    // [ R ]
-                    else if left.end == right.end {
-                        intersection.push(left.clone());
-                        self.ranges.remove(left_i);
-                        other.ranges.remove(right_i);
-                    }
-                    // [  L  ]
-                    // [ R ]
-                    else if left.end > right.end {
-                        intersection.push(right.clone());
-                        left.start = right.end;
-                        other.ranges.remove(right_i);
-                    }
+                // [ L ]
+                // [  R  ]
+                Ordering::Equal if left.end < right.end => {
+                    intersection.push(left.start..left.end);
+                    right.start = left.end;
+                    self.ranges.remove(left_i);
                 }
+                // [ L ]
+                // [ R ]
+                Ordering::Equal if left.end == right.end => {
+                    intersection.push(left.clone());
+                    self.ranges.remove(left_i);
+                    other.ranges.remove(right_i);
+                }
+                // [  L  ]
+                // [ R ]
+                Ordering::Equal if left.end > right.end => {
+                    intersection.push(right.clone());
+                    left.start = right.end;
+                    other.ranges.remove(right_i);
+                }
+                Ordering::Equal => {}
                 Ordering::Greater => {
                     //     [ L ]
                     // [ R ]
@@ -271,17 +272,43 @@ impl CharacterSet {
                 }
             }
         }
-        CharacterSet {
+        Self {
             ranges: intersection,
         }
     }
 
-    pub fn iter<'a>(&'a self) -> impl Iterator<Item = u32> + 'a {
-        self.ranges.iter().flat_map(|r| r.clone())
+    /// Produces a `CharacterSet` containing every character in `self` that is not present in
+    /// `other`.
+    pub fn difference(mut self, mut other: Self) -> Self {
+        self.remove_intersection(&mut other);
+        self
     }
 
-    pub fn chars<'a>(&'a self) -> impl Iterator<Item = char> + 'a {
-        self.iter().filter_map(char::from_u32)
+    /// Produces a `CharacterSet` containing every character that is in _exactly one_ of `self` or
+    /// `other`, but is not present in both sets.
+    pub fn symmetric_difference(mut self, mut other: Self) -> Self {
+        self.remove_intersection(&mut other);
+        self.add(&other)
+    }
+
+    pub fn char_codes(&self) -> impl Iterator<Item = u32> + '_ {
+        self.ranges.iter().flat_map(Clone::clone)
+    }
+
+    pub fn chars(&self) -> impl Iterator<Item = char> + '_ {
+        self.char_codes().filter_map(char::from_u32)
+    }
+
+    pub fn range_count(&self) -> usize {
+        self.ranges.len()
+    }
+
+    pub fn ranges(&self) -> impl Iterator<Item = RangeInclusive<char>> + '_ {
+        self.ranges.iter().filter_map(|range| {
+            let start = range.clone().find_map(char::from_u32)?;
+            let end = (range.start..range.end).rev().find_map(char::from_u32)?;
+            Some(start..=end)
+        })
     }
 
     pub fn is_empty(&self) -> bool {
@@ -290,53 +317,68 @@ impl CharacterSet {
 
     /// Get a reduced list of character ranges, assuming that a given
     /// set of characters can be safely ignored.
-    pub fn simplify_ignoring<'a>(
-        &'a self,
-        ruled_out_characters: &'a HashSet<u32>,
-    ) -> Vec<Range<char>> {
-        let mut prev_range: Option<Range<char>> = None;
-        self.chars()
-            .map(|c| (c, false))
-            .chain(Some(('\0', true)))
-            .filter_map(move |(c, done)| {
-                if done {
-                    return prev_range.clone();
-                }
-                if ruled_out_characters.contains(&(c as u32)) {
-                    return None;
-                }
-                if let Some(range) = prev_range.clone() {
-                    let mut prev_range_successor = range.end as u32 + 1;
-                    while prev_range_successor < c as u32 {
-                        if !ruled_out_characters.contains(&prev_range_successor) {
-                            prev_range = Some(c..c);
-                            return Some(range);
+    pub fn simplify_ignoring(&self, ruled_out_characters: &Self) -> Self {
+        let mut prev_range: Option<Range<u32>> = None;
+        Self {
+            ranges: self
+                .ranges
+                .iter()
+                .map(|range| Some(range.clone()))
+                .chain([None])
+                .filter_map(move |range| {
+                    if let Some(range) = &range {
+                        if ruled_out_characters.contains_codepoint_range(range.clone()) {
+                            return None;
+                        }
+
+                        if let Some(prev_range) = &mut prev_range {
+                            if ruled_out_characters
+                                .contains_codepoint_range(prev_range.end..range.start)
+                            {
+                                prev_range.end = range.end;
+                                return None;
+                            }
                         }
-                        prev_range_successor += 1;
                     }
-                    prev_range = Some(range.start..c);
-                    None
-                } else {
-                    prev_range = Some(c..c);
-                    None
-                }
-            })
-            .collect()
+
+                    let result = prev_range.clone();
+                    prev_range = range;
+                    result
+                })
+                .collect(),
+        }
+    }
+
+    pub fn contains_codepoint_range(&self, seek_range: Range<u32>) -> bool {
+        let ix = match self.ranges.binary_search_by(|probe| {
+            if probe.end <= seek_range.start {
+                Ordering::Less
+            } else if probe.start > seek_range.start {
+                Ordering::Greater
+            } else {
+                Ordering::Equal
+            }
+        }) {
+            Ok(ix) | Err(ix) => ix,
+        };
+        self.ranges.get(ix).map_or(false, |range| {
+            range.start <= seek_range.start && range.end >= seek_range.end
+        })
     }
 
     pub fn contains(&self, c: char) -> bool {
-        self.ranges.iter().any(|r| r.contains(&(c as u32)))
+        self.contains_codepoint_range(c as u32..c as u32 + 1)
     }
 }
 
 impl Ord for CharacterSet {
-    fn cmp(&self, other: &CharacterSet) -> Ordering {
+    fn cmp(&self, other: &Self) -> Ordering {
         let count_cmp = self
             .ranges
             .iter()
-            .map(|r| r.len())
+            .map(ExactSizeIterator::len)
             .sum::<usize>()
-            .cmp(&other.ranges.iter().map(|r| r.len()).sum());
+            .cmp(&other.ranges.iter().map(ExactSizeIterator::len).sum());
         if count_cmp != Ordering::Equal {
             return count_cmp;
         }
@@ -354,12 +396,12 @@ impl Ord for CharacterSet {
                 }
             }
         }
-        return Ordering::Equal;
+        Ordering::Equal
     }
 }
 
 impl PartialOrd for CharacterSet {
-    fn partial_cmp(&self, other: &CharacterSet) -> Option<Ordering> {
+    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
         Some(self.cmp(other))
     }
 }
@@ -372,11 +414,11 @@ impl fmt::Debug for CharacterSet {
             write!(f, "^ ")?;
             set = set.negate();
         }
-        for (i, c) in set.chars().enumerate() {
+        for (i, range) in set.ranges().enumerate() {
             if i > 0 {
                 write!(f, ", ")?;
             }
-            write!(f, "{:?}", c)?;
+            write!(f, "{range:?}")?;
         }
         write!(f, "]")?;
         Ok(())
@@ -384,8 +426,8 @@ impl fmt::Debug for CharacterSet {
 }
 
 impl Nfa {
-    pub fn new() -> Self {
-        Nfa { states: Vec::new() }
+    pub const fn new() -> Self {
+        Self { states: Vec::new() }
     }
 
     pub fn last_state_id(&self) -> u32 {
@@ -395,9 +437,9 @@ impl Nfa {
 
 impl fmt::Debug for Nfa {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "Nfa {{ states: {{\n")?;
+        writeln!(f, "Nfa {{ states: {{")?;
         for (i, state) in self.states.iter().enumerate() {
-            write!(f, "  {}: {:?},\n", i, state)?;
+            writeln!(f, "  {i}: {state:?},")?;
         }
         write!(f, "}} }}")?;
         Ok(())
@@ -420,7 +462,7 @@ impl<'a> NfaCursor<'a> {
     }
 
     pub fn force_reset(&mut self, states: Vec<u32>) {
-        self.state_ids = states
+        self.state_ids = states;
     }
 
     pub fn transition_chars(&self) -> impl Iterator<Item = (&CharacterSet, bool)> {
@@ -450,7 +492,7 @@ impl<'a> NfaCursor<'a> {
     fn group_transitions<'b>(
         iter: impl Iterator<Item = (&'b CharacterSet, bool, i32, u32)>,
     ) -> Vec<NfaTransition> {
-        let mut result: Vec<NfaTransition> = Vec::new();
+        let mut result = Vec::<NfaTransition>::new();
         for (chars, is_sep, prec, state) in iter {
             let mut chars = chars.clone();
             let mut i = 0;
@@ -458,9 +500,8 @@ impl<'a> NfaCursor<'a> {
                 let intersection = result[i].characters.remove_intersection(&mut chars);
                 if !intersection.is_empty() {
                     let mut intersection_states = result[i].states.clone();
-                    match intersection_states.binary_search(&state) {
-                        Err(j) => intersection_states.insert(j, state),
-                        _ => {}
+                    if let Err(j) = intersection_states.binary_search(&state) {
+                        intersection_states.insert(j, state);
                     }
                     let intersection_transition = NfaTransition {
                         characters: intersection,
@@ -486,25 +527,25 @@ impl<'a> NfaCursor<'a> {
                 });
             }
         }
-        result.sort_unstable_by(|a, b| a.characters.cmp(&b.characters));
 
         let mut i = 0;
-        'i_loop: while i < result.len() {
+        while i < result.len() {
             for j in 0..i {
                 if result[j].states == result[i].states
                     && result[j].is_separator == result[i].is_separator
                     && result[j].precedence == result[i].precedence
                 {
-                    let mut characters = CharacterSet::empty();
-                    swap(&mut characters, &mut result[j].characters);
+                    let characters = mem::take(&mut result[j].characters);
                     result[j].characters = characters.add(&result[i].characters);
                     result.remove(i);
-                    continue 'i_loop;
+                    i -= 1;
+                    break;
                 }
             }
             i += 1;
         }
 
+        result.sort_unstable_by(|a, b| a.characters.cmp(&b.characters));
         result
     }
 
@@ -786,18 +827,18 @@ mod tests {
                     (CharacterSet::from_char('e'), false, 0, 2),
                 ],
                 vec![
-                    NfaTransition {
-                        characters: CharacterSet::empty().add_char('a').add_range('c', 'd'),
-                        precedence: 0,
-                        states: vec![1],
-                        is_separator: false,
-                    },
                     NfaTransition {
                         characters: CharacterSet::empty().add_char('b').add_char('e'),
                         precedence: 0,
                         states: vec![2],
                         is_separator: false,
                     },
+                    NfaTransition {
+                        characters: CharacterSet::empty().add_char('a').add_range('c', 'd'),
+                        precedence: 0,
+                        states: vec![1],
+                        is_separator: false,
+                    },
                 ],
             ),
         ];
@@ -810,14 +851,13 @@ mod tests {
                         .map(|(chars, is_sep, prec, state)| (chars, *is_sep, *prec, *state))
                 ),
                 row.1,
-                "row {}",
-                i
+                "row {i}",
             );
         }
     }
 
     #[test]
-    fn test_character_set_remove_intersection() {
+    fn test_character_set_intersection_difference_ops() {
         struct Row {
             left: CharacterSet,
             right: CharacterSet,
@@ -942,6 +982,24 @@ mod tests {
                 "row {}b: {:?} - {:?}",
                 i, row.right, row.left
             );
+
+            assert_eq!(
+                row.left.clone().difference(row.right.clone()),
+                row.left_only,
+                "row {}b: {:?} -- {:?}",
+                i,
+                row.left,
+                row.right
+            );
+
+            let symm_difference = row.left_only.clone().add(&row.right_only);
+            assert_eq!(
+                row.left.clone().symmetric_difference(row.right.clone()),
+                symm_difference,
+                "row {i}b: {:?} ~~ {:?}",
+                row.left,
+                row.right
+            );
         }
     }
 
@@ -1002,7 +1060,8 @@ mod tests {
     }
 
     #[test]
-    fn test_character_set_get_ranges() {
+    #[allow(clippy::single_range_in_vec_init)]
+    fn test_character_set_simplify_ignoring() {
         struct Row {
             chars: Vec<char>,
             ruled_out_chars: Vec<char>,
@@ -1025,24 +1084,46 @@ mod tests {
                 ruled_out_chars: vec!['d', 'f', 'g'],
                 expected_ranges: vec!['a'..'h', 'z'..'z'],
             },
+            Row {
+                chars: vec!['a', 'b', 'c', 'g', 'h', 'i'],
+                ruled_out_chars: vec!['d', 'j'],
+                expected_ranges: vec!['a'..'c', 'g'..'i'],
+            },
+            Row {
+                chars: vec!['c', 'd', 'e', 'g', 'h'],
+                ruled_out_chars: vec!['a', 'b', 'c', 'd', 'e', 'f'],
+                expected_ranges: vec!['g'..'h'],
+            },
+            Row {
+                chars: vec!['I', 'N'],
+                ruled_out_chars: vec!['A', 'I', 'N', 'Z'],
+                expected_ranges: vec![],
+            },
         ];
 
         for Row {
             chars,
             ruled_out_chars,
             expected_ranges,
-        } in table.iter()
+        } in &table
         {
             let ruled_out_chars = ruled_out_chars
-                .into_iter()
-                .map(|c: &char| *c as u32)
-                .collect();
+                .iter()
+                .fold(CharacterSet::empty(), |set, c| set.add_char(*c));
             let mut set = CharacterSet::empty();
             for c in chars {
                 set = set.add_char(*c);
             }
-            let ranges = set.simplify_ignoring(&ruled_out_chars);
-            assert_eq!(ranges, *expected_ranges);
+            let actual = set.simplify_ignoring(&ruled_out_chars);
+            let expected = expected_ranges
+                .iter()
+                .fold(CharacterSet::empty(), |set, range| {
+                    set.add_range(range.start, range.end)
+                });
+            assert_eq!(
+                actual, expected,
+                "chars: {chars:?}, ruled out chars: {ruled_out_chars:?}"
+            );
         }
     }
 }
