@@ -1,3 +1,5 @@
+#![doc = include_str!("../README.md")]
+
 pub mod generate;
 pub mod highlight;
 pub mod logger;
@@ -8,8 +10,13 @@ pub mod query_testing;
 pub mod tags;
 pub mod test;
 pub mod test_highlight;
+pub mod test_tags;
 pub mod util;
 pub mod wasm;
 
 #[cfg(test)]
 mod tests;
+
+// To run compile fail tests
+#[cfg(doctest)]
+mod tests;
