@@ -1,12 +1,16 @@
-use super::helpers::edits::get_random_edit;
-use super::helpers::fixtures::{fixtures_dir, get_language, get_test_language};
-use super::helpers::random::Rand;
-use crate::generate::generate_parser_for_grammar;
-use crate::parse::perform_edit;
-use std::fs;
 use tree_sitter::{Node, Parser, Point, Tree};
 
-const JSON_EXAMPLE: &'static str = r#"
+use super::helpers::{
+    edits::get_random_edit,
+    fixtures::{fixtures_dir, get_language, get_test_language},
+    random::Rand,
+};
+use crate::{
+    generate::{generate_parser_for_grammar, load_grammar_file},
+    parse::perform_edit,
+};
+
+const JSON_EXAMPLE: &str = r#"
 
 [
   123,
@@ -17,7 +21,7 @@ const JSON_EXAMPLE: &'static str = r#"
 ]
 "#;
 
-const GRAMMAR_WITH_ALIASES_AND_EXTRAS: &'static str = r#"{
+const GRAMMAR_WITH_ALIASES_AND_EXTRAS: &str = r#"{
   "name": "aliases_and_extras",
 
   "extras": [
@@ -60,8 +64,8 @@ fn test_node_child() {
 
     assert_eq!(array_node.kind(), "array");
     assert_eq!(array_node.named_child_count(), 3);
-    assert_eq!(array_node.start_byte(), JSON_EXAMPLE.find("[").unwrap());
-    assert_eq!(array_node.end_byte(), JSON_EXAMPLE.find("]").unwrap() + 1);
+    assert_eq!(array_node.start_byte(), JSON_EXAMPLE.find('[').unwrap());
+    assert_eq!(array_node.end_byte(), JSON_EXAMPLE.find(']').unwrap() + 1);
     assert_eq!(array_node.start_position(), Point::new(2, 0));
     assert_eq!(array_node.end_position(), Point::new(8, 1));
     assert_eq!(array_node.child_count(), 7);
@@ -82,13 +86,13 @@ fn test_node_child() {
     assert_eq!(object_node.kind(), "object");
     assert_eq!(right_bracket_node.kind(), "]");
 
-    assert_eq!(left_bracket_node.is_named(), false);
-    assert_eq!(number_node.is_named(), true);
-    assert_eq!(comma_node1.is_named(), false);
-    assert_eq!(false_node.is_named(), true);
-    assert_eq!(comma_node2.is_named(), false);
-    assert_eq!(object_node.is_named(), true);
-    assert_eq!(right_bracket_node.is_named(), false);
+    assert!(!left_bracket_node.is_named());
+    assert!(number_node.is_named());
+    assert!(!comma_node1.is_named());
+    assert!(false_node.is_named());
+    assert!(!comma_node2.is_named());
+    assert!(object_node.is_named());
+    assert!(!right_bracket_node.is_named());
 
     assert_eq!(number_node.start_byte(), JSON_EXAMPLE.find("123").unwrap());
     assert_eq!(
@@ -106,7 +110,7 @@ fn test_node_child() {
     assert_eq!(false_node.start_position(), Point::new(4, 2));
     assert_eq!(false_node.end_position(), Point::new(4, 7));
 
-    assert_eq!(object_node.start_byte(), JSON_EXAMPLE.find("{").unwrap());
+    assert_eq!(object_node.start_byte(), JSON_EXAMPLE.find('{').unwrap());
     assert_eq!(object_node.start_position(), Point::new(5, 2));
     assert_eq!(object_node.end_position(), Point::new(7, 3));
 
@@ -119,9 +123,9 @@ fn test_node_child() {
     assert_eq!(pair_node.kind(), "pair");
     assert_eq!(right_brace_node.kind(), "}");
 
-    assert_eq!(left_brace_node.is_named(), false);
-    assert_eq!(pair_node.is_named(), true);
-    assert_eq!(right_brace_node.is_named(), false);
+    assert!(!left_brace_node.is_named());
+    assert!(pair_node.is_named());
+    assert!(!right_brace_node.is_named());
 
     assert_eq!(pair_node.start_byte(), JSON_EXAMPLE.find("\"x\"").unwrap());
     assert_eq!(pair_node.end_byte(), JSON_EXAMPLE.find("null").unwrap() + 4);
@@ -137,9 +141,9 @@ fn test_node_child() {
     assert_eq!(colon_node.kind(), ":");
     assert_eq!(null_node.kind(), "null");
 
-    assert_eq!(string_node.is_named(), true);
-    assert_eq!(colon_node.is_named(), false);
-    assert_eq!(null_node.is_named(), true);
+    assert!(string_node.is_named());
+    assert!(!colon_node.is_named());
+    assert!(null_node.is_named());
 
     assert_eq!(
         string_node.start_byte(),
@@ -165,6 +169,22 @@ fn test_node_child() {
     assert_eq!(object_node.parent().unwrap(), array_node);
     assert_eq!(array_node.parent().unwrap(), tree.root_node());
     assert_eq!(tree.root_node().parent(), None);
+
+    assert_eq!(
+        tree.root_node()
+            .child_containing_descendant(null_node)
+            .unwrap(),
+        array_node
+    );
+    assert_eq!(
+        array_node.child_containing_descendant(null_node).unwrap(),
+        object_node
+    );
+    assert_eq!(
+        object_node.child_containing_descendant(null_node).unwrap(),
+        pair_node
+    );
+    assert_eq!(pair_node.child_containing_descendant(null_node), None);
 }
 
 #[test]
@@ -202,7 +222,7 @@ fn test_node_children() {
 #[test]
 fn test_node_children_by_field_name() {
     let mut parser = Parser::new();
-    parser.set_language(get_language("python")).unwrap();
+    parser.set_language(&get_language("python")).unwrap();
     let source = "
         if one:
             a()
@@ -230,7 +250,7 @@ fn test_node_children_by_field_name() {
 #[test]
 fn test_node_parent_of_child_by_field_name() {
     let mut parser = Parser::new();
-    parser.set_language(get_language("javascript")).unwrap();
+    parser.set_language(&get_language("javascript")).unwrap();
     let tree = parser.parse("foo(a().b[0].c.d.e())", None).unwrap();
     let call_node = tree
         .root_node()
@@ -248,16 +268,44 @@ fn test_node_parent_of_child_by_field_name() {
     );
 }
 
+#[test]
+fn test_parent_of_zero_width_node() {
+    let code = "def dupa(foo):";
+
+    let mut parser = Parser::new();
+    parser.set_language(&get_language("python")).unwrap();
+
+    let tree = parser.parse(code, None).unwrap();
+    let root = tree.root_node();
+    let function_definition = root.child(0).unwrap();
+    let block = function_definition.child(4).unwrap();
+    let block_parent = block.parent().unwrap();
+
+    assert_eq!(block.to_string(), "(block)");
+    assert_eq!(block_parent.kind(), "function_definition");
+    assert_eq!(block_parent.to_string(), "(function_definition name: (identifier) parameters: (parameters (identifier)) body: (block))");
+
+    assert_eq!(
+        root.child_containing_descendant(block).unwrap(),
+        function_definition
+    );
+    assert_eq!(function_definition.child_containing_descendant(block), None);
+}
+
 #[test]
 fn test_node_field_name_for_child() {
     let mut parser = Parser::new();
-    parser.set_language(get_language("c")).unwrap();
-    let tree = parser.parse("x + y;", None).unwrap();
+    parser.set_language(&get_language("c")).unwrap();
+    let tree = parser
+        .parse("int w = x + /* y is special! */ y;", None)
+        .unwrap();
     let translation_unit_node = tree.root_node();
-    let binary_expression_node = translation_unit_node
-        .named_child(0)
+    let declaration_node = translation_unit_node.named_child(0).unwrap();
+
+    let binary_expression_node = declaration_node
+        .child_by_field_name("declarator")
         .unwrap()
-        .named_child(0)
+        .child_by_field_name("value")
         .unwrap();
 
     assert_eq!(binary_expression_node.field_name_for_child(0), Some("left"));
@@ -265,18 +313,20 @@ fn test_node_field_name_for_child() {
         binary_expression_node.field_name_for_child(1),
         Some("operator")
     );
+    // The comment should not have a field name, as it's just an extra
+    assert_eq!(binary_expression_node.field_name_for_child(2), None);
     assert_eq!(
-        binary_expression_node.field_name_for_child(2),
+        binary_expression_node.field_name_for_child(3),
         Some("right")
     );
     // Negative test - Not a valid child index
-    assert_eq!(binary_expression_node.field_name_for_child(3), None);
+    assert_eq!(binary_expression_node.field_name_for_child(4), None);
 }
 
 #[test]
 fn test_node_child_by_field_name_with_extra_hidden_children() {
     let mut parser = Parser::new();
-    parser.set_language(get_language("python")).unwrap();
+    parser.set_language(&get_language("python")).unwrap();
 
     // In the Python grammar, some fields are applied to `suite` nodes,
     // which consist of an invisible `indent` token followed by a block.
@@ -319,7 +369,7 @@ fn test_node_named_child() {
     assert_eq!(false_node.end_position(), Point::new(4, 7));
 
     assert_eq!(object_node.kind(), "object");
-    assert_eq!(object_node.start_byte(), JSON_EXAMPLE.find("{").unwrap());
+    assert_eq!(object_node.start_byte(), JSON_EXAMPLE.find('{').unwrap());
     assert_eq!(object_node.start_position(), Point::new(5, 2));
     assert_eq!(object_node.end_position(), Point::new(7, 3));
 
@@ -362,6 +412,22 @@ fn test_node_named_child() {
     assert_eq!(object_node.parent().unwrap(), array_node);
     assert_eq!(array_node.parent().unwrap(), tree.root_node());
     assert_eq!(tree.root_node().parent(), None);
+
+    assert_eq!(
+        tree.root_node()
+            .child_containing_descendant(null_node)
+            .unwrap(),
+        array_node
+    );
+    assert_eq!(
+        array_node.child_containing_descendant(null_node).unwrap(),
+        object_node
+    );
+    assert_eq!(
+        object_node.child_containing_descendant(null_node).unwrap(),
+        pair_node
+    );
+    assert_eq!(pair_node.child_containing_descendant(null_node), None);
 }
 
 #[test]
@@ -371,7 +437,7 @@ fn test_node_named_child_with_aliases_and_extras() {
 
     let mut parser = Parser::new();
     parser
-        .set_language(get_test_language(&parser_name, &parser_code, None))
+        .set_language(&get_test_language(&parser_name, &parser_code, None))
         .unwrap();
 
     let tree = parser.parse("b ... b ... c", None).unwrap();
@@ -385,13 +451,55 @@ fn test_node_named_child_with_aliases_and_extras() {
     assert_eq!(root.named_child(4).unwrap().kind(), "C");
 }
 
+#[test]
+fn test_node_descendant_count() {
+    let tree = parse_json_example();
+    let value_node = tree.root_node();
+    let all_nodes = get_all_nodes(&tree);
+
+    assert_eq!(value_node.descendant_count(), all_nodes.len());
+
+    let mut cursor = value_node.walk();
+    for (i, node) in all_nodes.iter().enumerate() {
+        cursor.goto_descendant(i);
+        assert_eq!(cursor.node(), *node, "index {i}");
+    }
+
+    for (i, node) in all_nodes.iter().enumerate().rev() {
+        cursor.goto_descendant(i);
+        assert_eq!(cursor.node(), *node, "rev index {i}");
+    }
+}
+
+#[test]
+fn test_descendant_count_single_node_tree() {
+    let mut parser = Parser::new();
+    parser
+        .set_language(&get_language("embedded-template"))
+        .unwrap();
+    let tree = parser.parse("hello", None).unwrap();
+
+    let nodes = get_all_nodes(&tree);
+    assert_eq!(nodes.len(), 2);
+    assert_eq!(tree.root_node().descendant_count(), 2);
+
+    let mut cursor = tree.root_node().walk();
+
+    cursor.goto_descendant(0);
+    assert_eq!(cursor.depth(), 0);
+    assert_eq!(cursor.node(), nodes[0]);
+    cursor.goto_descendant(1);
+    assert_eq!(cursor.depth(), 1);
+    assert_eq!(cursor.node(), nodes[1]);
+}
+
 #[test]
 fn test_node_descendant_for_range() {
     let tree = parse_json_example();
-    let array_node = tree.root_node().child(0).unwrap();
+    let array_node = tree.root_node();
 
     // Leaf node exactly matches the given bounds - byte query
-    let colon_index = JSON_EXAMPLE.find(":").unwrap();
+    let colon_index = JSON_EXAMPLE.find(':').unwrap();
     let colon_node = array_node
         .descendant_for_byte_range(colon_index, colon_index + 1)
         .unwrap();
@@ -412,7 +520,7 @@ fn test_node_descendant_for_range() {
     assert_eq!(colon_node.end_position(), Point::new(6, 8));
 
     // The given point is between two adjacent leaf nodes - byte query
-    let colon_index = JSON_EXAMPLE.find(":").unwrap();
+    let colon_index = JSON_EXAMPLE.find(':').unwrap();
     let colon_node = array_node
         .descendant_for_byte_range(colon_index, colon_index)
         .unwrap();
@@ -506,10 +614,10 @@ fn test_node_edit() {
     for _ in 0..10 {
         let mut nodes_before = get_all_nodes(&tree);
 
-        let edit = get_random_edit(&mut rand, &mut code);
+        let edit = get_random_edit(&mut rand, &code);
         let mut tree2 = tree.clone();
-        let edit = perform_edit(&mut tree2, &mut code, &edit);
-        for node in nodes_before.iter_mut() {
+        let edit = perform_edit(&mut tree2, &mut code, &edit).unwrap();
+        for node in &mut nodes_before {
             node.edit(&edit);
         }
 
@@ -532,7 +640,7 @@ fn test_node_edit() {
 #[test]
 fn test_root_node_with_offset() {
     let mut parser = Parser::new();
-    parser.set_language(get_language("javascript")).unwrap();
+    parser.set_language(&get_language("javascript")).unwrap();
     let tree = parser.parse("  if (a) b", None).unwrap();
 
     let node = tree.root_node_with_offset(6, Point::new(2, 2));
@@ -560,7 +668,7 @@ fn test_root_node_with_offset() {
 #[test]
 fn test_node_is_extra() {
     let mut parser = Parser::new();
-    parser.set_language(get_language("javascript")).unwrap();
+    parser.set_language(&get_language("javascript")).unwrap();
     let tree = parser.parse("foo(/* hi */);", None).unwrap();
 
     let root_node = tree.root_node();
@@ -575,7 +683,7 @@ fn test_node_is_extra() {
 #[test]
 fn test_node_sexp() {
     let mut parser = Parser::new();
-    parser.set_language(get_language("javascript")).unwrap();
+    parser.set_language(&get_language("javascript")).unwrap();
     let tree = parser.parse("if (a) b", None).unwrap();
     let root_node = tree.root_node();
     let if_node = root_node.descendant_for_byte_range(0, 0).unwrap();
@@ -664,7 +772,7 @@ fn test_node_field_names() {
 
     let mut parser = Parser::new();
     let language = get_test_language(&parser_name, &parser_code, None);
-    parser.set_language(language).unwrap();
+    parser.set_language(&language).unwrap();
 
     let tree = parser
         .parse("child-0 child-1 child-2 child-3 child-4", None)
@@ -734,7 +842,7 @@ fn test_node_field_calls_in_language_without_fields() {
 
     let mut parser = Parser::new();
     let language = get_test_language(&parser_name, &parser_code, None);
-    parser.set_language(language).unwrap();
+    parser.set_language(&language).unwrap();
 
     let tree = parser.parse("b c d", None).unwrap();
 
@@ -744,26 +852,26 @@ fn test_node_field_calls_in_language_without_fields() {
 
     let mut cursor = root_node.walk();
     assert_eq!(cursor.field_name(), None);
-    assert_eq!(cursor.goto_first_child(), true);
+    assert!(cursor.goto_first_child());
     assert_eq!(cursor.field_name(), None);
 }
 
 #[test]
 fn test_node_is_named_but_aliased_as_anonymous() {
-    let (parser_name, parser_code) = generate_parser_for_grammar(
-        &fs::read_to_string(
-            &fixtures_dir()
-                .join("test_grammars")
-                .join("named_rule_aliased_as_anonymous")
-                .join("grammar.json"),
-        )
-        .unwrap(),
+    let grammar_json = load_grammar_file(
+        &fixtures_dir()
+            .join("test_grammars")
+            .join("named_rule_aliased_as_anonymous")
+            .join("grammar.js"),
+        None,
     )
     .unwrap();
 
+    let (parser_name, parser_code) = generate_parser_for_grammar(&grammar_json).unwrap();
+
     let mut parser = Parser::new();
     let language = get_test_language(&parser_name, &parser_code, None);
-    parser.set_language(language).unwrap();
+    parser.set_language(&language).unwrap();
 
     let tree = parser.parse("B C B", None).unwrap();
 
@@ -782,13 +890,14 @@ fn test_node_is_named_but_aliased_as_anonymous() {
 #[test]
 fn test_node_numeric_symbols_respect_simple_aliases() {
     let mut parser = Parser::new();
-    parser.set_language(get_language("python")).unwrap();
+    parser.set_language(&get_language("python")).unwrap();
 
     // Example 1:
-    // Python argument lists can contain "splat" arguments, which are not allowed within
-    // other expressions. This includes `parenthesized_list_splat` nodes like `(*b)`. These
-    // `parenthesized_list_splat` nodes are aliased as `parenthesized_expression`. Their numeric
-    // `symbol`, aka `kind_id` should match that of a normal `parenthesized_expression`.
+    // Python argument lists can contain "splat" arguments, which are not allowed
+    // within other expressions. This includes `parenthesized_list_splat` nodes
+    // like `(*b)`. These `parenthesized_list_splat` nodes are aliased as
+    // `parenthesized_expression`. Their numeric `symbol`, aka `kind_id` should
+    // match that of a normal `parenthesized_expression`.
     let tree = parser.parse("(a((*b)))", None).unwrap();
     let root = tree.root_node();
     assert_eq!(
@@ -810,10 +919,10 @@ fn test_node_numeric_symbols_respect_simple_aliases() {
     assert_eq!(inner_expr_node.kind_id(), outer_expr_node.kind_id());
 
     // Example 2:
-    // Ruby handles the unary (negative) and binary (minus) `-` operators using two different
-    // tokens. One or more of these is an external token that's aliased as `-`. Their numeric
-    // kind ids should match.
-    parser.set_language(get_language("ruby")).unwrap();
+    // Ruby handles the unary (negative) and binary (minus) `-` operators using two
+    // different tokens. One or more of these is an external token that's
+    // aliased as `-`. Their numeric kind ids should match.
+    parser.set_language(&get_language("ruby")).unwrap();
     let tree = parser.parse("-a - b", None).unwrap();
     let root = tree.root_node();
     assert_eq!(
@@ -841,22 +950,22 @@ fn get_all_nodes(tree: &Tree) -> Vec<Node> {
     let mut visited_children = false;
     let mut cursor = tree.walk();
     loop {
-        result.push(cursor.node());
-        if !visited_children && cursor.goto_first_child() {
-            continue;
+        if !visited_children {
+            result.push(cursor.node());
+            if !cursor.goto_first_child() {
+                visited_children = true;
+            }
         } else if cursor.goto_next_sibling() {
             visited_children = false;
-        } else if cursor.goto_parent() {
-            visited_children = true;
-        } else {
+        } else if !cursor.goto_parent() {
             break;
         }
     }
-    return result;
+    result
 }
 
 fn parse_json_example() -> Tree {
     let mut parser = Parser::new();
-    parser.set_language(get_language("json")).unwrap();
+    parser.set_language(&get_language("json")).unwrap();
     parser.parse(JSON_EXAMPLE, None).unwrap()
 }
