@@ -1,6 +1,7 @@
-use super::FREE_FN;
 use std::os::raw::c_void;
 
+use super::FREE_FN;
+
 /// A raw pointer and a length, exposed as an iterator.
 pub struct CBufferIter<T> {
     ptr: *mut T,
@@ -9,7 +10,7 @@ pub struct CBufferIter<T> {
 }
 
 impl<T> CBufferIter<T> {
-    pub unsafe fn new(ptr: *mut T, count: usize) -> Self {
+    pub const unsafe fn new(ptr: *mut T, count: usize) -> Self {
         Self { ptr, count, i: 0 }
     }
 }
@@ -23,7 +24,7 @@ impl<T: Copy> Iterator for CBufferIter<T> {
             None
         } else {
             self.i += 1;
-            Some(unsafe { *self.ptr.offset(i as isize) })
+            Some(unsafe { *self.ptr.add(i) })
         }
     }
 
@@ -37,6 +38,8 @@ impl<T: Copy> ExactSizeIterator for CBufferIter<T> {}
 
 impl<T> Drop for CBufferIter<T> {
     fn drop(&mut self) {
-        unsafe { (FREE_FN)(self.ptr as *mut c_void) };
+        if !self.ptr.is_null() {
+            unsafe { (FREE_FN)(self.ptr.cast::<c_void>()) };
+        }
     }
 }
