@@ -6,17 +6,14 @@ const path = require('path');
 const inputFiles = [
   'binding.c',
   'binding.js',
-  'exports.json',
+  'exports.txt',
   'imports.js',
   'prefix.js',
   ...list('../include/tree_sitter'),
-  ...list('../src')
-]
+  ...list('../src'),
+];
 
-const outputFiles = [
-  'tree-sitter.js',
-  'tree-sitter.wasm',
-]
+const outputFiles = ['tree-sitter.js', 'tree-sitter.wasm'];
 
 const outputMtime = Math.min(...outputFiles.map(mtime));
 
@@ -28,9 +25,10 @@ for (const inputFile of inputFiles) {
 }
 
 function list(dir) {
-  return fs.readdirSync(path.join(__dirname, dir), 'utf8')
-    .filter(p => !p.startsWith('.'))
-    .map(p => path.join(dir, p));
+  return fs
+    .readdirSync(path.join(__dirname, dir), 'utf8')
+    .filter((p) => !p.startsWith('.'))
+    .map((p) => path.join(dir, p));
 }
 
 function mtime(p) {
